# Core AWS Service abstractions
#import socks.ssm
import boto3
import socks.athena
import socks.cloudwatch
import socks.dynamodb
import socks.s3
import socks.secretsmanager
import socks.sqs
import socks.ssm
import socks.sts
import socks.aws_lambda
